# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for cidr_merge filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.filter.cidr_merge import _cidr_merge


INVALID_DATA_MERGE = ["0.1234.34.44", "1.00000.2.000.22"]

VALID_DATA_MEREGE = ["192.168.0.0/17", "192.168.128.0/17", "192.168.128.1"]

VALID_OUTPUT_MERGE = ["192.168.0.0/16"]

VALID_DATA_SPAN = ["192.168.1.1", "192.168.1.2", "192.168.1.3", "192.168.1.4"]

VALID_OUTPUT_SPAN = "192.168.1.0/29"


class TestCidrMerge(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_data_merge(self):
        """Check passing invalid argspec"""

        args = ["", INVALID_DATA_MERGE, "merge"]
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            _cidr_merge(*args, **kwargs)
        self.assertIn("invalid IPNetwork 0.1234.34.44", str(error.exception))

    def test_valid_data_merge(self):
        """test for cidr_merge plugin with merge"""

        args = ["", VALID_DATA_MEREGE, "merge"]
        result = _cidr_merge(*args)
        self.assertEqual(result, VALID_OUTPUT_MERGE)

    def test_valid_data_span(self):
        """test for cidr_merge plugin with span"""

        args = ["", VALID_DATA_SPAN, "span"]
        result = _cidr_merge(*args)
        self.assertEqual(result, VALID_OUTPUT_SPAN)

    def test_valid_data_with_invalid_action(self):
        """Check passing valid data as per criteria"""

        args = ["", VALID_DATA_SPAN, "span1"]
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            _cidr_merge(*args, **kwargs)
        self.assertIn("cidr_merge: invalid action 'span1'", str(error.exception))



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for ipwrap filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

import pytest

from ansible.errors import AnsibleFilterError
from ansible.template import AnsibleUndefined

from ansible_collections.ansible.utils.plugins.filter.ipv6 import _ipv6


VALID_DATA = [
    "192.24.2.1",
    "::ffff:192.168.32.0/120",
    "",
    "::ffff:192.24.2.1/128",
    "192.168.32.0/24",
    "fe80::100/10",
    True,
]


VALID_OUTPUT = [
    "::ffff:192.168.32.0/120",
    "::ffff:192.24.2.1/128",
    "fe80::100/10",
]

VALID_OUTPUT1 = ["192.168.32.0/24", "192.24.2.1/32"]

VALID_OUTPUT2 = ["::ffff:192.168.32.0", "::ffff:192.24.2.1", "fe80::100"]


class TestIp6(unittest.TestCase):
    def setUp(self):
        pass

    def test_ipv6_undefined_value(self):
        """Check ipv6 filter undefined value"""
        args = ["", AnsibleUndefined(name="my_ip"), ""]
        with pytest.raises(
            AnsibleFilterError,
            match="Unrecognized type <<class 'ansible.template.AnsibleUndefined'>> for ipv6 filter <value>",
        ):
            _ipv6(*args)

    def test_ipv6_filter_empty_query(self):
        """Check ipv6 filter empty query"""
        args = ["", VALID_DATA, ""]
        result = _ipv6(*args)
        self.assertEqual(result, VALID_OUTPUT)

    def test_ipv6_ipv4_conversion(self):
        """Check ipv6 to ipv4 conversion"""
        args = ["", VALID_DATA, "ipv4"]
        result = _ipv6(*args)
        self.assertEqual(result, VALID_OUTPUT1)

    def test_ipv6_filter_address_query(self):
        """Check ipv6 filter address query"""
        args = ["", VALID_DATA, "address"]
        result = _ipv6(*args)
        self.assertEqual(result, VALID_OUTPUT2)



from __future__ import absolute_import, division, print_function


__metaclass__ = type
from ansible.utils.path import unfrackpath
from ansible_collections.ansible.netcommon.tests.unit.compat.mock import MagicMock


mock_unfrackpath_noop = MagicMock(spec_set=unfrackpath, side_effect=lambda x, *args, **kwargs: x)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.filter.param_list_compare import param_list_compare


class TestParam_list_compare_merge(unittest.TestCase):
    def test_valid_data(self):
        """Check passing valid data as per criteria"""

        base = ["interfaces", "l2_interfaces", "l3_interfaces"]
        other = ["all"]
        args = [base, other]
        kwargs = {}
        result = param_list_compare(*args, **kwargs)
        self.assertEqual(result["actionable"], base)

    def test_valid_data_same_contents(self):
        """Check passing valid data as per criteria"""

        base = ["interfaces", "l2_interfaces", "l3_interfaces"]
        other = ["interfaces", "l2_interfaces", "l3_interfaces"]
        args = [base, other]
        kwargs = {}
        result = param_list_compare(*args, **kwargs)
        self.assertEqual(result["actionable"], base)

    def test_valid_data_with_not_bang(self):
        """Check passing valid data as per criteria"""

        base = ["interfaces", "l2_interfaces", "l3_interfaces"]
        other = ["!l2_interfaces", "all"]
        args = [base, other]
        expected = ["interfaces", "l3_interfaces"]
        kwargs = {}
        result = param_list_compare(*args, **kwargs)
        self.assertEqual(result["actionable"], expected)

    def test_invalid_args_length_data(self):
        """Check passing valid data as per criteria"""

        base = {}
        args = [base]
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            param_list_compare(*args, **kwargs)
        self.assertIn(
            "Missing either 'base' or 'other value in filter input",
            str(error.exception),
        )

    def test_invalid_base_type_data(self):
        """Check passing valid data as per criteria"""

        base = {}
        other = ["all"]
        args = [base, other]
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            param_list_compare(*args, **kwargs)
        self.assertIn("cannot be converted to a list", str(error.exception))

    def test_invalid_other_type_data(self):
        """Check passing valid data as per criteria"""

        base = ["interfaces"]
        other = {"all": None}
        args = [base, other]
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            param_list_compare(*args, **kwargs)
        self.assertIn("cannot be converted to a list", str(error.exception))

    def test_invalid_unsupported_bang(self):
        """Check passing valid data as per criteria"""

        base = ["interfaces"]
        other = ["every"]
        args = [base, other]
        kwargs = {}
        result = param_list_compare(*args, **kwargs)
        self.assertEqual(result["unsupported"], other)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible.errors import AnsibleError, AnsibleFilterError

from ansible_collections.ansible.utils.plugins.filter.to_xml import _to_xml


INVALID_DATA = '<netconf-state xmlns="urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring">'

VALID_DATA = {
    "interface-configurations": {
        "@xmlns": "http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg",
        "key1": "value1",
    },
}

OUTPUT_TABS = """<?xml version="1.0" encoding="utf-8"?>
<interface-configurations xmlns="http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg">
\t<key1>value1</key1>
</interface-configurations>"""

OUTPUT_SPACES = """<?xml version="1.0" encoding="utf-8"?>
<interface-configurations xmlns="http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg">
    <key1>value1</key1>
</interface-configurations>"""


class TestToXml(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_data(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = ["", INVALID_DATA, "xmltodict"]
        kwargs = {}
        with self.assertRaises(AnsibleError) as error:
            _to_xml(*args, **kwargs)
        self.assertIn("we were unable to convert to dict", str(error.exception))

    def test_valid_data(self):
        """Check passing valid data as per criteria"""
        self.maxDiff = None
        args = ["", VALID_DATA, "xmltodict"]
        result = _to_xml(*args)
        self.assertEqual(result, OUTPUT_TABS)

    def test_args(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = []
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            _to_xml(*args, **kwargs)
        self.assertIn("missing required arguments: data", str(error.exception))

    def test_invalid_engine(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = ["", VALID_DATA, "test"]
        kwargs = {}
        with self.assertRaises(AnsibleError) as error:
            _to_xml(*args, **kwargs)
        self.assertIn("engine: test is not supported", str(error.exception))

    def test_indent_with_spaces(self):
        """Check passing indent with spaces and default indent_width"""
        self.maxDiff = None
        args = ["", VALID_DATA, "xmltodict", "spaces", 4]
        result = _to_xml(*args)
        self.assertEqual(result, OUTPUT_SPACES)

    def test_invalid_indent(self):
        """Check passing invalid indent value"""

        # missing required arguments
        args = ["", VALID_DATA, "xmltodict", "test"]
        kwargs = {}
        with self.assertRaises(AnsibleError) as error:
            _to_xml(*args, **kwargs)
        self.assertIn(
            "value of indent must be one of: tabs, spaces, got: test",
            str(error.exception),
        )



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for ipwrap filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

import pytest

from ansible_collections.ansible.utils.plugins.filter.ipsubnet import _ipsubnet


address = "192.168.144.5"
subnet = "192.168.0.0/16"


class TestIpSubnet(unittest.TestCase):
    def setUp(self):
        pass

    def test_ipvsubnet_address_subnet(self):
        """convert address to subnet"""
        args = ["", address, ""]
        result = _ipsubnet(*args)
        self.assertEqual(result, "192.168.144.5/32")

    def test_ipvsubnet_filter_subnet(self):
        """check if a given string is a subnet"""
        args = ["", subnet, ""]
        result = _ipsubnet(*args)
        self.assertEqual(result, "192.168.0.0/16")

    def test_ipvsubnet_filter_subnet_size(self):
        """Get the number of subnets a given subnet can be split into."""
        args = ["", subnet, "20"]
        result = _ipsubnet(*args)
        self.assertEqual(result, "16")

    def test_ipvsubnet_filter_subnet_with_1st_index(self):
        """Get the 1st subnet"""
        args = ["", subnet, "20", 0]
        result = _ipsubnet(*args)
        self.assertEqual(result, "192.168.0.0/20")

    def test_ipvsubnet_filter_subnet_with_last_index(self):
        """Get the last subnet"""
        args = ["", subnet, "20", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "192.168.240.0/20")

    def test_ipvsubnet_filter_address_with_size(self):
        """Get biggest subnet that contains that given IP address"""
        args = ["", address, "20"]
        result = _ipsubnet(*args)
        self.assertEqual(result, "192.168.144.0/20")

    def test_ipvsubnet_filter_address_with_1st_index(self):
        """Get the 1st subnet"""
        args = ["", address, "18", 0]
        result = _ipsubnet(*args)
        self.assertEqual(result, "192.168.128.0/18")

    def test_ipvsubnet_filter_address_with_last_index(self):
        """Get the last subnet"""
        args = ["", address, "18", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "192.168.144.4/31")

    def test_ipvsubnet_filter_lots_of_subnets_v4_bottom1(self):
        """Use the subnets call to see if it slows way down with v4"""
        args = ["", "0.0.0.0/1", "31", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "127.255.255.254/31")

    def test_ipvsubnet_filter_lots_of_subnets_v4_bottom2(self):
        """Use the subnets call to see if it slows way down with v4"""
        args = ["", "1.0.0.0/1", "3", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "96.0.0.0/3")

    def test_ipvsubnet_filter_lots_of_subnets_v4_top1(self):
        """Use the subnets call to see if it slows way down with v4"""
        args = ["", "128.0.0.0/1", "31", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "255.255.255.254/31")

    def test_ipvsubnet_filter_lots_of_subnets_v4_top2(self):
        """Use the subnets call to see if it slows way down with v4"""
        args = ["", "130.0.0.0/1", "31", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "255.255.255.254/31")

    def test_ipvsubnet_filter_lots_of_subnets_v6_bottom1(self):
        """Use the subnets call to see if it slows way down with v6"""
        args = ["", "8000::/1", "127", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "ffff:ffff:ffff:ffff:ffff:ffff:ffff:fffe/127")

    def test_ipvsubnet_filter_lots_of_subnets_v6_bottom2(self):
        """Use the subnets call to see if it slows way down with v6"""
        args = ["", "9000::/1", "127", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "ffff:ffff:ffff:ffff:ffff:ffff:ffff:fffe/127")

    def test_ipvsubnet_filter_lots_of_subnets_v6_top1(self):
        """Use the subnets call to see if it slows way down with v6"""
        args = ["", "::/1", "127", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "7fff:ffff:ffff:ffff:ffff:ffff:ffff:fffe/127")

    def test_ipvsubnet_filter_lots_of_subnets_v6_top2(self):
        """Use the subnets call to see if it slows way down with v6"""
        args = ["", "1000::/1", "127", -1]
        result = _ipsubnet(*args)
        self.assertEqual(result, "7fff:ffff:ffff:ffff:ffff:ffff:ffff:fffe/127")

    def test_ipvsubnet_filter_rank_address_in_subnet(self):
        """The rank of the IP in the subnet (the IP is the 36870nth /32 of the subnet)"""
        args = ["", address, subnet]
        result = _ipsubnet(*args)
        self.assertEqual(result, "36870")

    def test_ipvsubnet_filter_rank_address_in_subnet1(self):
        """The rank of the IP in the 192.168.144.0/24"""
        args = ["", address, "192.168.144.0/24"]
        result = _ipsubnet(*args)
        self.assertEqual(result, "6")

    def test_ipvsubnet_filter_rank_address_in_subnet2(self):
        """The rank of the IP in the 192.168.144.0/24"""
        args = ["", "192.168.144.1/30", "192.168.144.0/24"]
        result = _ipsubnet(*args)
        self.assertEqual(result, "1")

    def test_ipvsubnet_filter_rank_address_in_subnet3(self):
        """The rank of the IP in the 192.168.144.0/24"""
        args = ["", "192.168.144.16/30", "192.168.144.0/24"]
        result = _ipsubnet(*args)
        self.assertEqual(result, "5")


@pytest.mark.parametrize(
    "test_case,expected",
    [
        [["2600:1f1c:1b3:8f00::/56", 64, 0], "2600:1f1c:1b3:8f00::/64"],
        [["2600:1f1c:1b3:8f00::/56", "64", "1"], "2600:1f1c:1b3:8f01::/64"],
        [["2600:1f1c:1b3:8f00::/56", "64", "2"], "2600:1f1c:1b3:8f02::/64"],
        [["2600:1f1c:1b3:8f00::/56", "64", "-2"], "2600:1f1c:1b3:8ffe::/64"],
        [["2600:1f1c:1b3:8f00::/56", "64", "-1"], "2600:1f1c:1b3:8fff::/64"],
        [["2600:1f1c:1b3:8f00::/56", "65", "0"], "2600:1f1c:1b3:8f00::/65"],
        [["2600:1f1c:1b3:8f00::/56", "65", "1"], "2600:1f1c:1b3:8f00:8000::/65"],
        [["2600:1f1c:1b3:8f00::/56", "65", "2"], "2600:1f1c:1b3:8f01::/65"],
        [["2600:1f1c:1b3:8f00::/56", "65", "-2"], "2600:1f1c:1b3:8fff::/65"],
        [["2600:1f1c:1b3:8f00::/56", "65", "-1"], "2600:1f1c:1b3:8fff:8000::/65"],
    ],
)
def test_ipvsubnet_get_subnet(test_case, expected):
    assert _ipsubnet("", *test_case) == expected


@pytest.mark.parametrize(
    "test_case",
    [
        ["2600:1f1c:1b3:8f00::/56", "64", "257"],  # subnet id too big for /64
        ["2600:1f1c:1b3:8f00:::/56", "64", "0"],  # Too many colons
        ["2600:1f1c:1b3:8f00::/56", "129", "0"],  # Must be /128 or less
    ],
)
def test_ipvsubnet_get_subnet_fail(test_case):
    assert _ipsubnet("", *test_case) is False



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for ipwrap filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible_collections.ansible.utils.plugins.filter.next_nth_usable import _next_nth_usable


class Test_Next_Nth_Usable(unittest.TestCase):
    def setUp(self):
        pass

    def test_next_nth_usable_filter(self):
        """next_nth_usable filter"""
        args = ["", "192.168.122.1/24", 2]
        result = _next_nth_usable(*args)
        self.assertEqual(result, "192.168.122.3")

    def test_next_nth_usable_with_empty_return_string(self):
        """Check ipv4 to ipv6 conversion"""
        args = ["", "192.168.122.254/24", 2]
        result = _next_nth_usable(*args)
        self.assertEqual(result, None)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for slaac filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible_collections.ansible.utils.plugins.filter.slaac import _slaac


class Test_slaac(unittest.TestCase):
    def setUp(self):
        pass

    def test_slaac_filter_1(self):
        """slaac filter"""
        args = [
            "",
            "fdcf:1894:23b5:d38c:0000:0000:0000:0000",
            "c2:31:b3:83:bf:2b",
        ]
        result = _slaac(*args)
        self.assertEqual(result, "fdcf:1894:23b5:d38c:c031:b3ff:fe83:bf2b")



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for nthhost filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible_collections.ansible.utils.plugins.filter.nthhost import _nthhost


class Test_nthhost(unittest.TestCase):
    def setUp(self):
        pass

    def test_nthhost_filter_1(self):
        """nthhost filter"""
        args = ["", "10.0.0.0/8", "305"]
        result = _nthhost(*args)
        self.assertEqual(result, "10.0.1.49")

    def test_nthhost_filter_2(self):
        """nthhost filter"""
        args = ["", "10.0.0.0/8", "-1"]
        result = _nthhost(*args)
        self.assertEqual(result, "10.255.255.255")



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.filter.consolidate import _consolidate


class TestConsolidate(unittest.TestCase):
    def setUp(self):
        pass

    def test_consolidate_plugin(self):
        data_sources = [
            {
                "data": [
                    {
                        "duplex": "auto",
                        "enabled": True,
                        "name": "GigabitEthernet0/0",
                        "note": ["Connected green wire"],
                        "speed": "auto",
                    },
                    {
                        "description": "Configured by Ansible - Interface 1",
                        "duplex": "auto",
                        "enabled": True,
                        "mtu": 1500,
                        "name": "GigabitEthernet0/1",
                        "note": ["Connected blue wire", "Configured by Paul"],
                        "speed": "auto",
                        "vifs": [
                            {
                                "comment": "Needs reconfiguration",
                                "description": "Eth1 - VIF 100",
                                "enabled": True,
                                "mtu": 400,
                                "vlan_id": 100,
                            },
                            {
                                "description": "Eth1 - VIF 101",
                                "enabled": True,
                                "vlan_id": 101,
                            },
                        ],
                    },
                    {
                        "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
                        "enabled": False,
                        "mtu": 600,
                        "name": "GigabitEthernet0/2",
                    },
                ],
                "match_key": "name",
                "name": "interfaces",
            },
            {
                "data": [
                    {"name": "GigabitEthernet0/0"},
                    {
                        "mode": "access",
                        "name": "GigabitEthernet0/1",
                        "trunk": {
                            "allowed_vlans": [
                                "11",
                                "12",
                                "59",
                                "67",
                                "75",
                                "77",
                                "81",
                                "100",
                                "400-408",
                                "411-413",
                                "415",
                                "418",
                                "982",
                                "986",
                                "988",
                                "993",
                            ],
                        },
                    },
                    {
                        "mode": "trunk",
                        "name": "GigabitEthernet0/2",
                        "trunk": {
                            "allowed_vlans": [
                                "11",
                                "12",
                                "59",
                                "67",
                                "75",
                                "77",
                                "81",
                                "100",
                                "400-408",
                                "411-413",
                                "415",
                                "418",
                                "982",
                                "986",
                                "988",
                                "993",
                            ],
                            "encapsulation": "dot1q",
                        },
                    },
                ],
                "match_key": "name",
                "name": "l2_interfaces",
            },
            {
                "data": [
                    {
                        "ipv4": [{"address": "192.168.0.2/24"}],
                        "name": "GigabitEthernet0/0",
                    },
                    {"name": "GigabitEthernet0/1"},
                    {"name": "GigabitEthernet0/2"},
                    {"name": "Loopback888"},
                    {"name": "Loopback999"},
                ],
                "match_key": "name",
                "name": "l3_interfaces",
            },
        ]

        output = {
            "GigabitEthernet0/0": {
                "interfaces": {
                    "duplex": "auto",
                    "enabled": True,
                    "name": "GigabitEthernet0/0",
                    "note": ["Connected green wire"],
                    "speed": "auto",
                },
                "l2_interfaces": {"name": "GigabitEthernet0/0"},
                "l3_interfaces": {
                    "ipv4": [{"address": "192.168.0.2/24"}],
                    "name": "GigabitEthernet0/0",
                },
            },
            "GigabitEthernet0/1": {
                "interfaces": {
                    "description": "Configured by Ansible - Interface 1",
                    "duplex": "auto",
                    "enabled": True,
                    "mtu": 1500,
                    "name": "GigabitEthernet0/1",
                    "note": ["Connected blue wire", "Configured by Paul"],
                    "speed": "auto",
                    "vifs": [
                        {
                            "comment": "Needs reconfiguration",
                            "description": "Eth1 - VIF 100",
                            "enabled": True,
                            "mtu": 400,
                            "vlan_id": 100,
                        },
                        {
                            "description": "Eth1 - VIF 101",
                            "enabled": True,
                            "vlan_id": 101,
                        },
                    ],
                },
                "l2_interfaces": {
                    "mode": "access",
                    "name": "GigabitEthernet0/1",
                    "trunk": {
                        "allowed_vlans": [
                            "11",
                            "12",
                            "59",
                            "67",
                            "75",
                            "77",
                            "81",
                            "100",
                            "400-408",
                            "411-413",
                            "415",
                            "418",
                            "982",
                            "986",
                            "988",
                            "993",
                        ],
                    },
                },
                "l3_interfaces": {"name": "GigabitEthernet0/1"},
            },
            "GigabitEthernet0/2": {
                "interfaces": {
                    "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
                    "enabled": False,
                    "mtu": 600,
                    "name": "GigabitEthernet0/2",
                },
                "l2_interfaces": {
                    "mode": "trunk",
                    "name": "GigabitEthernet0/2",
                    "trunk": {
                        "allowed_vlans": [
                            "11",
                            "12",
                            "59",
                            "67",
                            "75",
                            "77",
                            "81",
                            "100",
                            "400-408",
                            "411-413",
                            "415",
                            "418",
                            "982",
                            "986",
                            "988",
                            "993",
                        ],
                        "encapsulation": "dot1q",
                    },
                },
                "l3_interfaces": {"name": "GigabitEthernet0/2"},
            },
            "Loopback888": {
                "interfaces": {},
                "l2_interfaces": {},
                "l3_interfaces": {"name": "Loopback888"},
            },
            "Loopback999": {
                "interfaces": {},
                "l2_interfaces": {},
                "l3_interfaces": {"name": "Loopback999"},
            },
        }
        fail_missing_match_value = False
        fail_missing_match_key = False
        fail_duplicate = False
        args = [
            "",
            data_sources,
            fail_missing_match_key,
            fail_missing_match_value,
            fail_duplicate,
        ]

        result = _consolidate(*args)
        self.assertEqual(result, output)

    def test_fail_missing_match_key(self):
        data_sources = [
            {
                "data": [
                    {
                        "duplex": "auto",
                        "enabled": True,
                        "name": "GigabitEthernet0/0",
                        "note": ["Connected green wire"],
                        "speed": "auto",
                    },
                    {
                        "description": "Configured by Ansible - Interface 1",
                        "duplex": "auto",
                        "enabled": True,
                        "mtu": 1500,
                        "name": "GigabitEthernet0/1",
                        "note": ["Connected blue wire", "Configured by Paul"],
                        "speed": "auto",
                        "vifs": [
                            {
                                "comment": "Needs reconfiguration",
                                "description": "Eth1 - VIF 100",
                                "enabled": True,
                                "mtu": 400,
                                "vlan_id": 100,
                            },
                            {
                                "description": "Eth1 - VIF 101",
                                "enabled": True,
                                "vlan_id": 101,
                            },
                        ],
                    },
                    {
                        "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
                        "enabled": False,
                        "mtu": 600,
                        "name": "GigabitEthernet0/2",
                    },
                ],
                "match_key": "name",
                "name": "interfaces",
            },
            {
                "data": [
                    {"name": "GigabitEthernet0/0"},
                    {
                        "mode": "access",
                        "name": "GigabitEthernet0/1",
                        "trunk": {
                            "allowed_vlans": [
                                "11",
                                "12",
                                "59",
                                "67",
                                "75",
                                "77",
                                "81",
                                "100",
                                "400-408",
                                "411-413",
                                "415",
                                "418",
                                "982",
                                "986",
                                "988",
                                "993",
                            ],
                        },
                    },
                    {
                        "mode": "trunk",
                        "name": "GigabitEthernet0/2",
                        "trunk": {
                            "allowed_vlans": [
                                "11",
                                "12",
                                "59",
                                "67",
                                "75",
                                "77",
                                "81",
                                "100",
                                "400-408",
                                "411-413",
                                "415",
                                "418",
                                "982",
                                "986",
                                "988",
                                "993",
                            ],
                            "encapsulation": "dot1q",
                        },
                    },
                ],
                "match_key": "name",
                "name": "l2_interfaces",
            },
            {
                "data": [
                    {
                        "ipv4": [{"address": "192.168.0.2/24"}],
                        "intf_name": "GigabitEthernet0/0",
                    },
                    {"name": "GigabitEthernet0/1"},
                    {"name": "GigabitEthernet0/2"},
                    {"name": "Loopback888"},
                    {"name": "Loopback999"},
                ],
                "match_key": "name",
                "name": "l3_interfaces",
            },
        ]

        fail_missing_match_key = True
        args = ["", data_sources, fail_missing_match_key, False, False]
        with self.assertRaises(AnsibleFilterError) as error:
            _consolidate(*args)
        self.assertIn(
            "Error when using plugin 'consolidate': 'fail_missing_match_key' reported missing match key 'name' in data source 3 in list entry 1",
            str(error.exception),
        )

    def test_fail_duplicate(self):
        data_sources = [
            {
                "data": [
                    {
                        "duplex": "auto",
                        "enabled": True,
                        "name": "GigabitEthernet0/0",
                        "note": ["Connected green wire"],
                        "speed": "auto",
                    },
                    {
                        "description": "Configured by Ansible - Interface 1",
                        "duplex": "auto",
                        "enabled": True,
                        "mtu": 1500,
                        "name": "GigabitEthernet0/1",
                        "note": ["Connected blue wire", "Configured by Paul"],
                        "speed": "auto",
                        "vifs": [
                            {
                                "comment": "Needs reconfiguration",
                                "description": "Eth1 - VIF 100",
                                "enabled": True,
                                "mtu": 400,
                                "vlan_id": 100,
                            },
                            {
                                "description": "Eth1 - VIF 101",
                                "enabled": True,
                                "vlan_id": 101,
                            },
                        ],
                    },
                    {
                        "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
                        "enabled": False,
                        "mtu": 600,
                        "name": "GigabitEthernet0/2",
                    },
                ],
                "match_key": "name",
                "name": "interfaces",
            },
            {
                "data": [
                    {"name": "GigabitEthernet0/0"},
                    {
                        "mode": "access",
                        "name": "GigabitEthernet0/1",
                        "trunk": {
                            "allowed_vlans": [
                                "11",
                                "12",
                                "59",
                                "67",
                                "75",
                                "77",
                                "81",
                                "100",
                                "400-408",
                                "411-413",
                                "415",
                                "418",
                                "982",
                                "986",
                                "988",
                                "993",
                            ],
                        },
                    },
                    {
                        "mode": "trunk",
                        "name": "GigabitEthernet0/2",
                        "trunk": {
                            "allowed_vlans": [
                                "11",
                                "12",
                                "59",
                                "67",
                                "75",
                                "77",
                                "81",
                                "100",
                                "400-408",
                                "411-413",
                                "415",
                                "418",
                                "982",
                                "986",
                                "988",
                                "993",
                            ],
                            "encapsulation": "dot1q",
                        },
                    },
                ],
                "match_key": "name",
                "name": "l2_interfaces",
            },
            {
                "data": [
                    {
                        "ipv4": [{"address": "192.168.0.2/24"}],
                        "name": "GigabitEthernet0/0",
                    },
                    {
                        "ipv4": [{"address": "192.168.0.3/24"}],
                        "name": "GigabitEthernet0/0",
                    },
                    {"name": "GigabitEthernet0/1"},
                    {"name": "GigabitEthernet0/2"},
                    {"name": "Loopback888"},
                    {"name": "Loopback999"},
                ],
                "match_key": "name",
                "name": "l3_interfaces",
            },
        ]

        fail_missing_match_value = False
        fail_missing_match_key = False
        fail_duplicate = True
        args = [
            "",
            data_sources,
            fail_missing_match_key,
            fail_missing_match_value,
            fail_duplicate,
        ]
        with self.assertRaises(AnsibleFilterError) as error:
            _consolidate(*args)
        self.assertIn(
            "Error when using plugin 'consolidate': 'fail_duplicate' reported duplicate values in data source 3",
            str(error.exception),
        )



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for ipaddr filter plugins
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

import pytest

from ansible.errors import AnsibleFilterError
from ansible.template import AnsibleUndefined

from ansible_collections.ansible.utils.plugins.filter.cidr_merge import cidr_merge
from ansible_collections.ansible.utils.plugins.filter.ip4_hex import ip4_hex
from ansible_collections.ansible.utils.plugins.filter.ipaddr import _ipaddr
from ansible_collections.ansible.utils.plugins.filter.ipmath import ipmath
from ansible_collections.ansible.utils.plugins.filter.ipsubnet import ipsubnet
from ansible_collections.ansible.utils.plugins.filter.network_in_network import network_in_network
from ansible_collections.ansible.utils.plugins.filter.network_in_usable import network_in_usable
from ansible_collections.ansible.utils.plugins.filter.next_nth_usable import next_nth_usable
from ansible_collections.ansible.utils.plugins.filter.nthhost import nthhost
from ansible_collections.ansible.utils.plugins.filter.previous_nth_usable import previous_nth_usable
from ansible_collections.ansible.utils.plugins.filter.reduce_on_network import reduce_on_network
from ansible_collections.ansible.utils.plugins.filter.slaac import slaac
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddr_utils import ipaddr


netaddr = pytest.importorskip("netaddr")


class TestIpFilter(unittest.TestCase):
    def test_cidr_merge(self):
        with pytest.raises(AnsibleFilterError, match="cidr_merge: expected iterable, got None"):
            cidr_merge(None)

        with pytest.raises(AnsibleFilterError, match="cidr_merge: invalid action 'floop'"):
            cidr_merge([], "floop")

        self.assertEqual(cidr_merge([]), [])
        self.assertEqual(cidr_merge([], "span"), None)
        subnets = ["1.12.1.0/24"]
        self.assertEqual(cidr_merge(subnets), subnets)
        self.assertEqual(cidr_merge(subnets, "span"), subnets[0])
        subnets = ["1.12.1.0/25", "1.12.1.128/25"]
        self.assertEqual(cidr_merge(subnets), ["1.12.1.0/24"])
        self.assertEqual(cidr_merge(subnets, "span"), "1.12.1.0/24")
        subnets = ["1.12.1.0/25", "1.12.1.128/25", "1.12.2.0/24"]
        self.assertEqual(cidr_merge(subnets), ["1.12.1.0/24", "1.12.2.0/24"])
        self.assertEqual(cidr_merge(subnets, "span"), "1.12.0.0/22")
        subnets = ["1.12.1.1", "1.12.1.255"]
        self.assertEqual(cidr_merge(subnets), ["1.12.1.1/32", "1.12.1.255/32"])
        self.assertEqual(cidr_merge(subnets, "span"), "1.12.1.0/24")

    def test_ipaddr_undefined_value(self):
        """Check ipaddr filter undefined value"""
        args = ["", AnsibleUndefined(name="my_ip"), ""]
        with pytest.raises(
            AnsibleFilterError,
            match="Unrecognized type <<class 'ansible.template.AnsibleUndefined'>> for ipaddr filter <value>",
        ):
            _ipaddr(*args)

    def test_ipaddr_empty_query(self):
        self.assertEqual(ipaddr("192.0.2.230"), "192.0.2.230")
        self.assertEqual(ipaddr("192.0.2.230/30"), "192.0.2.230/30")
        self.assertEqual(ipaddr([]), [])

        self.assertEqual(ipaddr(True), False)
        self.assertEqual(ipaddr(""), False)

    def test_ipaddr_6to4_query(self):
        v6_address = "2002:c000:02e6::1/48"
        self.assertEqual(ipaddr("192.0.2.230", "6to4"), v6_address)
        self.assertEqual(ipaddr("192.0.2.230/24", "6to4"), v6_address)
        self.assertFalse(ipaddr("192.0.2.0/24", "6to4"))

        self.assertFalse(ipaddr("fd::e", "6to4"))
        self.assertFalse(ipaddr("fd::e/20", "6to4"))

        self.assertEqual(ipaddr("2002:c000:02e6::1", "6to4"), "2002:c000:02e6::1")
        self.assertEqual(ipaddr(v6_address, "6to4"), v6_address)
        self.assertEqual(
            ipaddr(
                ["192.0.2.230", "192.0.2.0/24", "fd::e", "2002:c000:02e6::1"],
                "6to4",
            ),
            [v6_address, "2002:c000:02e6::1"],
        )

    def test_ipaddr_address_query(self):
        self.assertEqual(ipaddr("192.0.2.230", "address"), "192.0.2.230")
        self.assertEqual(ipaddr("192.0.2.230/24", "address"), "192.0.2.230")
        self.assertIsNone(ipaddr("192.0.2.0/24", "address"))
        self.assertEqual(ipaddr("192.0.2.0/31", "address"), "192.0.2.0")
        self.assertEqual(ipaddr("2001::1", "address"), "2001::1")
        self.assertEqual(ipaddr("2001::1/48", "address"), "2001::1")
        self.assertEqual(ipaddr("2001::", "address"), "2001::")
        self.assertEqual(ipaddr("2001::/48", "address"), "2001::")

    def test_ipaddr_bool_query(self):
        self.assertTrue(ipaddr("192.0.2.20", "bool"))
        self.assertFalse(ipaddr("192.900.2.20", "bool"))

    def test_netmask(self):
        address = "1.1.1.1/24"
        self.assertEqual(ipaddr(address, "netmask"), "255.255.255.0")
        address = "1.1.1.1/25"
        self.assertEqual(ipaddr(address, "netmask"), "255.255.255.128")
        address = "1.12.1.34/32"
        self.assertEqual(ipaddr(address, "netmask"), "255.255.255.255")

    def test_network(self):
        address = "1.12.1.34/32"
        self.assertEqual(ipaddr(address, "network"), "1.12.1.34")
        address = "1.12.1.34/255.255.255.255"
        self.assertEqual(ipaddr(address, "network"), "1.12.1.34")
        address = "1.12.1.34"
        self.assertEqual(ipaddr(address, "network"), "1.12.1.34")
        address = "1.12.1.35/31"
        self.assertEqual(ipaddr(address, "network"), "1.12.1.34")
        address = "1.12.1.34/24"
        self.assertEqual(ipaddr(address, "network"), "1.12.1.0")

    def test_broadcast(self):
        address = "1.12.1.34/24"
        self.assertEqual(ipaddr(address, "broadcast"), "1.12.1.255")
        address = "1.12.1.34/16"
        self.assertEqual(ipaddr(address, "broadcast"), "1.12.255.255")
        address = "1.12.1.34/27"
        self.assertEqual(ipaddr(address, "broadcast"), "1.12.1.63")
        address = "1.12.1.34/32"
        self.assertEqual(ipaddr(address, "broadcast"), None)
        address = "1.12.1.35/31"
        self.assertEqual(ipaddr(address, "broadcast"), None)

    def test_first_usable(self):
        with pytest.raises(AnsibleFilterError, match="Not a network address"):
            ipaddr("1.12.1.34", "first_usable")

        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "first_usable"), "1.12.1.1")
        address = "1.12.1.36/24"
        self.assertEqual(ipaddr(address, "first_usable"), "1.12.1.1")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "first_usable"), "1.12.1.33")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "first_usable"), "1.12.1.33")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "first_usable"), "1.12.1.36")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "first_usable"), "1.12.1.36")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "first_usable"), None)

    def test_host_query(self):
        self.assertEqual(ipaddr("192.0.2.1", "host"), "192.0.2.1/32")
        address = "192.0.2.12/20"
        self.assertEqual(ipaddr(address, "host"), address)
        address = "192.0.2.0/31"
        self.assertEqual(ipaddr(address, "host"), address)
        self.assertIsNone(ipaddr("192.0.2.0/24", "host"))

    def test_last_usable(self):
        with pytest.raises(AnsibleFilterError, match="Not a network address"):
            ipaddr("1.12.1.34", "last_usable")

        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "last_usable"), "1.12.1.254")
        address = "1.12.1.36/24"
        self.assertEqual(ipaddr(address, "last_usable"), "1.12.1.254")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "last_usable"), "1.12.1.46")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "last_usable"), "1.12.1.46")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "last_usable"), "1.12.1.37")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "last_usable"), "1.12.1.37")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "last_usable"), None)

    def test_wildcard(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "wildcard"), "0.0.0.255")
        address = "1.12.1.0/25"
        self.assertEqual(ipaddr(address, "wildcard"), "0.0.0.127")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "wildcard"), "0.0.0.15")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "wildcard"), "0.0.0.15")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "wildcard"), "0.0.0.1")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "wildcard"), "0.0.0.1")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "wildcard"), "0.0.0.0")
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "wildcard"), "0.0.0.255")

    def test_size_usable(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "size_usable"), 254)
        address = "1.12.1.0/25"
        self.assertEqual(ipaddr(address, "size_usable"), 126)
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "size_usable"), 14)
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "size_usable"), 14)
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "size_usable"), 2)
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "size_usable"), 2)
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "size_usable"), 0)
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "size_usable"), 254)

    def test_ipaddr_public_query(self):
        self.assertIsNone(ipaddr("192.168.1.12", "public"))
        self.assertIsNone(ipaddr("127.0.1.25", "public"))
        self.assertIsNone(ipaddr("255.255.240.0", "public"))
        self.assertEqual(ipaddr("76.120.99.190", "public"), "76.120.99.190")
        self.assertEqual(
            ipaddr(["192.168.1.12", "127.0.1.25", "255.255.240.0"], "public"),
            [],
        )
        self.assertEqual(
            ipaddr(
                [
                    "192.168.1.12",
                    "127.0.1.25",
                    "255.255.240.0",
                    "76.120.99.190",
                ],
                "public",
            ),
            ["76.120.99.190"],
        )

    def test_range_usable(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "range_usable"), "1.12.1.1-1.12.1.254")
        address = "1.12.1.0/25"
        self.assertEqual(ipaddr(address, "range_usable"), "1.12.1.1-1.12.1.126")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "range_usable"), "1.12.1.33-1.12.1.46")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "range_usable"), "1.12.1.33-1.12.1.46")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "range_usable"), "1.12.1.36-1.12.1.37")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "range_usable"), "1.12.1.36-1.12.1.37")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "range_usable"), None)
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "range_usable"), "1.12.1.1-1.12.1.254")

    def test_address_prefix(self):
        # Regular address
        address = "1.12.1.12/24"
        self.assertEqual(ipaddr(address, "address/prefix"), address)

        # Network address - invalid
        address = "1.12.1.0/24"
        self.assertFalse(ipaddr(address, "address/prefix"))
        # But valid in a /31
        address = "1.12.1.0/31"
        self.assertEqual(ipaddr(address, "address/prefix"), address)

        # Broadcast address - invalid
        address = "1.12.1.255/24"
        self.assertFalse(ipaddr(address, "address/prefix"))
        # But valid in a /31
        address = "1.12.1.255/31"
        self.assertEqual(ipaddr(address, "address/prefix"), address)

        # /32 - always valid?
        address = "1.12.1.0/32"
        self.assertEqual(ipaddr(address, "address/prefix"), address)
        address = "1.12.1.12/32"
        self.assertEqual(ipaddr(address, "address/prefix"), address)
        address = "1.12.1.255/32"
        self.assertEqual(ipaddr(address, "address/prefix"), address)

        # No prefix means /32
        address = "1.12.1.34"
        self.assertEqual(ipaddr(address, "address/prefix"), "1.12.1.34/32")

        # Hostmask also works
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "address/prefix"), "1.12.1.36/28")
        # But not hostmasks that don't make a valid CIDR prefix
        address = "1.12.1.36/255.255.255.88"
        self.assertFalse(ipaddr(address, "address/prefix"))

    def test_ip_prefix(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "ip/prefix"), None)
        address = "1.12.1.0/25"
        self.assertEqual(ipaddr(address, "ip/prefix"), None)
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "ip/prefix"), "1.12.1.36/28")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "ip/prefix"), "1.12.1.36/28")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "ip/prefix"), "1.12.1.36/31")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "ip/prefix"), "1.12.1.37/31")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "ip/prefix"), None)
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "ip/prefix"), "1.12.1.254/24")

    def test_ip_netmask(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "ip_netmask"), None)
        address = "1.12.1.0/25"
        self.assertEqual(ipaddr(address, "ip_netmask"), None)
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "ip_netmask"), "1.12.1.36 255.255.255.240")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "ip_netmask"), "1.12.1.36 255.255.255.240")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "ip_netmask"), "1.12.1.36 255.255.255.254")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "ip_netmask"), "1.12.1.37 255.255.255.254")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "ip_netmask"), None)
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "ip_netmask"), "1.12.1.254 255.255.255.0")

    def test_ipv6_query(self):
        self.assertEqual(ipaddr("fd00:123::97", "ipv6"), "fd00:123::97")
        self.assertEqual(ipaddr("192.0.2.230", "ipv6"), "::ffff:192.0.2.230/128")

    def test_ipaddr_link_local_query(self):
        self.assertEqual(ipaddr("169.254.0.12", "link-local"), "169.254.0.12")
        self.assertIsNone(ipaddr("192.0.2.12", "link-local"))
        self.assertEqual(ipaddr("fe80::9", "link-local"), "fe80::9")
        self.assertIsNone(ipaddr("2001::", "link-local"))

    def test_network_id(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "network_id"), "1.12.1.0")
        address = "1.12.1.0/25"
        self.assertEqual(ipaddr(address, "network_id"), "1.12.1.0")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "network_id"), "1.12.1.32")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "network_id"), "1.12.1.32")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "network_id"), "1.12.1.36")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "network_id"), "1.12.1.36")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "network_id"), "1.12.1.36")
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "network_id"), "1.12.1.0")

    def test_network_prefix(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "network/prefix"), "1.12.1.0/24")
        address = "1.12.1.0/25"
        self.assertEqual(ipaddr(address, "network/prefix"), "1.12.1.0/25")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "network/prefix"), "1.12.1.32/28")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "network/prefix"), "1.12.1.32/28")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "network/prefix"), "1.12.1.36/31")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "network/prefix"), "1.12.1.36/31")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "network/prefix"), "1.12.1.36/32")
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "network/prefix"), "1.12.1.0/24")

    def test_network_netmask(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "network_netmask"), "1.12.1.0 255.255.255.0")
        address = "1.12.1.0/25"
        self.assertEqual(ipaddr(address, "network_netmask"), "1.12.1.0 255.255.255.128")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "network_netmask"), "1.12.1.32 255.255.255.240")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "network_netmask"), "1.12.1.32 255.255.255.240")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "network_netmask"), "1.12.1.36 255.255.255.254")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "network_netmask"), "1.12.1.36 255.255.255.254")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "network_netmask"), "1.12.1.36 255.255.255.255")
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "network_netmask"), "1.12.1.0 255.255.255.0")

    def test_network_wildcard(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "network_wildcard"), "1.12.1.0 0.0.0.255")
        address = "1.12.1.0/25"
        self.assertEqual(ipaddr(address, "network_wildcard"), "1.12.1.0 0.0.0.127")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "network_wildcard"), "1.12.1.32 0.0.0.15")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "network_wildcard"), "1.12.1.32 0.0.0.15")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "network_wildcard"), "1.12.1.36 0.0.0.1")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "network_wildcard"), "1.12.1.36 0.0.0.1")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "network_wildcard"), "1.12.1.36 0.0.0.0")
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "network_wildcard"), "1.12.1.0 0.0.0.255")

    def test_next_usable(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "next_usable"), "1.12.1.1")
        address = "1.12.1.36/24"
        self.assertEqual(ipaddr(address, "next_usable"), "1.12.1.37")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "next_usable"), "1.12.1.37")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "next_usable"), "1.12.1.37")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "next_usable"), "1.12.1.37")
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "next_usable"), None)
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "next_usable"), None)
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "next_usable"), None)

    def test_peer(self):
        address = "1.12.1.0/31"
        self.assertEqual(ipaddr(address, "peer"), "1.12.1.1")
        address = "1.12.1.1/31"
        self.assertEqual(ipaddr(address, "peer"), "1.12.1.0")
        address = "1.12.1.1/30"
        self.assertEqual(ipaddr(address, "peer"), "1.12.1.2")
        address = "1.12.1.2/30"
        self.assertEqual(ipaddr(address, "peer"), "1.12.1.1")
        with self.assertRaises(AnsibleFilterError):
            address = "1.12.1.34"
            ipaddr(address, "peer")
        with self.assertRaises(AnsibleFilterError):
            address = "1.12.1.33/29"
            ipaddr(address, "peer")
        with self.assertRaises(AnsibleFilterError):
            address = "1.12.1.32/30"
            ipaddr(address, "peer")
        with self.assertRaises(AnsibleFilterError):
            address = "1.12.1.35/30"
            ipaddr(address, "peer")
        with self.assertRaises(AnsibleFilterError):
            address = "1.12.1.34/32"
            ipaddr(address, "peer")

    def test_previous_usable(self):
        address = "1.12.1.0/24"
        self.assertEqual(ipaddr(address, "previous_usable"), None)
        address = "1.12.1.36/24"
        self.assertEqual(ipaddr(address, "previous_usable"), "1.12.1.35")
        address = "1.12.1.36/28"
        self.assertEqual(ipaddr(address, "previous_usable"), "1.12.1.35")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(ipaddr(address, "previous_usable"), "1.12.1.35")
        address = "1.12.1.36/31"
        self.assertEqual(ipaddr(address, "previous_usable"), None)
        address = "1.12.1.37/31"
        self.assertEqual(ipaddr(address, "previous_usable"), "1.12.1.36")
        address = "1.12.1.36/32"
        self.assertEqual(ipaddr(address, "previous_usable"), None)
        address = "1.12.1.254/24"
        self.assertEqual(ipaddr(address, "previous_usable"), "1.12.1.253")

    def test_ipmath(self):
        self.assertEqual(ipmath("192.168.1.5", 5), "192.168.1.10")
        self.assertEqual(ipmath("192.168.1.5", -5), "192.168.1.0")
        self.assertEqual(ipmath("192.168.0.5", -10), "192.167.255.251")

        self.assertEqual(ipmath("192.168.1.1/24", 5), "192.168.1.6")
        self.assertEqual(ipmath("192.168.1.6/24", -5), "192.168.1.1")
        self.assertEqual(ipmath("192.168.2.6/24", -10), "192.168.1.252")

        self.assertEqual(ipmath("2001::1", 8), "2001::9")
        self.assertEqual(ipmath("2001::1", 9), "2001::a")
        self.assertEqual(ipmath("2001::1", 10), "2001::b")
        self.assertEqual(ipmath("2001::5", -3), "2001::2")
        self.assertEqual(ipmath("2001::5", -10), "2000:ffff:ffff:ffff:ffff:ffff:ffff:fffb")

        expected = "You must pass a valid IP address; invalid_ip is invalid"
        with self.assertRaises(AnsibleFilterError) as exc:
            ipmath("invalid_ip", 8)
        self.assertEqual(exc.exception.message, expected)

        expected = "You must pass an integer for arithmetic; some_number is not a valid integer"
        with self.assertRaises(AnsibleFilterError) as exc:
            ipmath("1.2.3.4", "some_number")
        self.assertEqual(exc.exception.message, expected)

    def test_ipsubnet(self):
        test_cases = (
            (("1.1.1.1/24", "30"), "64"),
            (("1.12.1.34/32", "1.12.1.34/24"), "35"),
            (("192.168.50.0/24", "192.168.0.0/16"), "51"),
            (("192.168.144.5", "192.168.0.0/16"), "36870"),
            (("192.168.144.5", "192.168.144.5/24"), "6"),
            (("192.168.144.5/32", "192.168.144.0/24"), "6"),
            (("192.168.144.16/30", "192.168.144.0/24"), "5"),
            (("192.168.144.5",), "192.168.144.5/32"),
            (("192.168.0.0/16",), "192.168.0.0/16"),
            (("192.168.144.5",), "192.168.144.5/32"),
            (("192.168.0.0/16", "20"), "16"),
            (("192.168.0.0/16", "20", "0"), "192.168.0.0/20"),
            (("192.168.0.0/16", "20", "-1"), "192.168.240.0/20"),
            (("192.168.0.0/16", "20", "5"), "192.168.80.0/20"),
            (("192.168.0.0/16", "20", "-5"), "192.168.176.0/20"),
            (("192.168.144.5", "20"), "192.168.144.0/20"),
            (("192.168.144.5", "18", "0"), "192.168.128.0/18"),
            (("192.168.144.5", "18", "-1"), "192.168.144.4/31"),
            (("192.168.144.5", "18", "5"), "192.168.144.0/23"),
            (("192.168.144.5", "18", "-5"), "192.168.144.0/27"),
            (("span", "test", "error"), False),
            (("test",), False),
            (("192.168.144.5", "500000", "-5"), False),
            (("192.168.144.5", "18", "500000"), False),
            (("200000", "18", "-5"), "0.3.13.64/27"),
        )
        for args, res in test_cases:
            self._test_ipsubnet(args, res)

    def _test_ipsubnet(self, ipsubnet_args, expected_result):
        self.assertEqual(ipsubnet(*ipsubnet_args), expected_result)

        expected = "Requested subnet size of 24 is invalid"
        with self.assertRaises(AnsibleFilterError) as exc:
            ipsubnet("1.1.1.1/25", "24")
        self.assertEqual(exc.exception.message, expected)

        with self.assertRaisesRegexp(
            AnsibleFilterError,
            "You must pass a valid subnet or IP address; invalid_subnet is invalid",
        ):
            ipsubnet("192.168.144.5", "invalid_subnet")

        with self.assertRaisesRegexp(
            AnsibleFilterError,
            "192.168.144.0/30 is not in the subnet 192.168.144.4/30",
        ):
            ipsubnet("192.168.144.1/30", "192.168.144.5/30")

    def test_nthhost(self):
        address = "1.12.1.0/24"
        self.assertFalse(nthhost(address))
        self.assertEqual(nthhost(address, 5), "1.12.1.5")
        address = "1.12.1.36/24"
        self.assertEqual(nthhost(address, 10), "1.12.1.10")
        address = "1.12.1.34"
        self.assertFalse(nthhost(address, "last_usable"), "Not a network address")
        address = "1.12.1.36/28"
        self.assertEqual(nthhost(address, 4), "1.12.1.36")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(nthhost(address, 4), "1.12.1.36")
        address = "1.12.1.36/31"
        self.assertEqual(nthhost(address, 1), "1.12.1.37")
        address = "1.12.1.37/31"
        self.assertEqual(nthhost(address, 1), "1.12.1.37")
        address = "1.12.1.36/32"
        self.assertFalse(nthhost(address, 1))
        address = "1.12.1.254/24"
        self.assertEqual(nthhost(address, 2), "1.12.1.2")

    def test_next_nth_usable(self):
        address = "1.12.1.0/24"
        self.assertEqual(next_nth_usable(address, 5), "1.12.1.5")
        address = "1.12.1.36/24"
        self.assertEqual(next_nth_usable(address, 10), "1.12.1.46")
        address = "1.12.1.36/28"
        self.assertEqual(next_nth_usable(address, 4), "1.12.1.40")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(next_nth_usable(address, 4), "1.12.1.40")
        address = "1.12.1.36/31"
        self.assertEqual(next_nth_usable(address, 1), "1.12.1.37")
        address = "1.12.1.37/31"
        self.assertEqual(next_nth_usable(address, 1), None)
        address = "1.12.1.36/32"
        self.assertEqual(next_nth_usable(address, 1), None)
        address = "1.12.1.254/24"
        self.assertEqual(next_nth_usable(address, 2), None)

    def test_previous_nth_usable(self):
        address = "1.12.1.0/24"
        self.assertEqual(previous_nth_usable(address, 5), None)
        address = "1.12.1.36/24"
        self.assertEqual(previous_nth_usable(address, 10), "1.12.1.26")
        address = "1.12.1.36/28"
        self.assertEqual(previous_nth_usable(address, 2), "1.12.1.34")
        address = "1.12.1.36/255.255.255.240"
        self.assertEqual(previous_nth_usable(address, 2), "1.12.1.34")
        address = "1.12.1.36/31"
        self.assertEqual(previous_nth_usable(address, 1), None)
        address = "1.12.1.37/31"
        self.assertEqual(previous_nth_usable(address, 1), "1.12.1.36")
        address = "1.12.1.36/32"
        self.assertEqual(previous_nth_usable(address, 1), None)
        address = "1.12.1.254/24"
        self.assertEqual(previous_nth_usable(address, 2), "1.12.1.252")

    def test_network_in_usable(self):
        subnet = "1.12.1.0/24"
        address = "1.12.1.10"
        self.assertEqual(network_in_usable(subnet, address), True)
        subnet = "1.12.1.0/24"
        address = "1.12.0.10"
        self.assertEqual(network_in_usable(subnet, address), False)
        subnet = "1.12.1.32/28"
        address = "1.12.1.36"
        self.assertEqual(network_in_usable(subnet, address), True)
        subnet = "1.12.1.32/28"
        address = "1.12.1.36/31"
        self.assertEqual(network_in_usable(subnet, address), True)
        subnet = "1.12.1.32/28"
        address = "1.12.1.48/31"
        self.assertEqual(network_in_usable(subnet, address), False)
        subnet = "1.12.1.32/255.255.255.240"
        address = "1.12.1.31"
        self.assertEqual(network_in_usable(subnet, address), False)
        subnet = "1.12.1.36/31"
        address = "1.12.1.36"
        self.assertEqual(network_in_usable(subnet, address), True)
        subnet = "1.12.1.37/31"
        address = "1.12.1.35"
        self.assertEqual(network_in_usable(subnet, address), False)
        subnet = "1.12.1.36/32"
        address = "1.12.1.36"
        self.assertEqual(network_in_usable(subnet, address), True)
        subnet = "1.12.1.0/24"
        address = "1.12.2.0"
        self.assertEqual(network_in_usable(subnet, address), False)

    def test_network_in_network(self):
        subnet = "1.12.1.0/24"
        address = "1.12.1.0"
        self.assertEqual(network_in_network(subnet, address), True)
        subnet = "1.12.1.0/24"
        address = "1.12.0.10"
        self.assertEqual(network_in_network(subnet, address), False)
        subnet = "1.12.1.32/28"
        address = "1.12.1.32/28"
        self.assertEqual(network_in_network(subnet, address), True)
        subnet = "1.12.1.32/28"
        address = "1.12.1.47"
        self.assertEqual(network_in_network(subnet, address), True)
        subnet = "1.12.1.32/28"
        address = "1.12.1.48/31"
        self.assertEqual(network_in_network(subnet, address), False)
        subnet = "1.12.1.32/255.255.255.240"
        address = "1.12.1.31"
        self.assertEqual(network_in_network(subnet, address), False)
        subnet = "1.12.1.36/31"
        address = "1.12.1.36"
        self.assertEqual(network_in_network(subnet, address), True)
        subnet = "1.12.1.37/31"
        address = "1.12.1.35"
        self.assertEqual(network_in_network(subnet, address), False)
        subnet = "1.12.1.36/32"
        address = "1.12.1.36"
        self.assertEqual(network_in_network(subnet, address), True)
        subnet = "1.12.1.0/24"
        address = "1.12.2.0"
        self.assertEqual(network_in_network(subnet, address), False)

    def test_reduce_on_network(self):
        subnet = "1.12.1.0/28"
        addresses = ["1.12.1.0", "1.12.0.10"]
        self.assertEqual(reduce_on_network(addresses, subnet), [addresses[0]])
        addresses = ["1.12.2.0", "1.12.1.236"]
        self.assertEqual(reduce_on_network(addresses, subnet), [])

        subnet = "1.12.1.32/28"
        addresses = ["1.12.1.32/28", "1.12.1.47", "1.12.1.48/31"]
        self.assertEqual(reduce_on_network(addresses, subnet), addresses[:-1])

        subnet = "1.12.1.36/32"
        addresses = ["1.12.1.31", "1.12.1.36", "1.12.1.35", "1.12.1.40"]
        self.assertEqual(reduce_on_network(addresses, subnet), [addresses[1]])
        subnet = "1.12.1.32/255.255.255.240"
        self.assertEqual(reduce_on_network(addresses, subnet), addresses[1:])

    def test_slaac(self):
        mac = "00:50:b6:aa:99:e2"
        self.assertFalse(slaac("192.168.1.20", mac))
        self.assertFalse(slaac("fd::9"))
        self.assertFalse(slaac("floop"))

        self.assertEqual(slaac("fd::9", mac), "fd::250:b6ff:feaa:99eb")
        self.assertEqual(
            slaac("fd00:1234:5678:9abc:def0::/20", mac),
            "fd00:1000::250:b6ff:feaa:99e2",
        )

    def test_ip4_hex(self):
        self.assertEqual(ip4_hex("192.0.2.24"), "c0000218")
        self.assertEqual(ip4_hex("192.0.2.24", "."), "c0.00.02.18")



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for ipwrap filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

import pytest

from ansible.errors import AnsibleFilterError
from ansible.template import AnsibleUndefined

from ansible_collections.ansible.utils.plugins.filter.ipwrap import _ipwrap


VALID_DATA = [
    "192.24.2.1",
    "host.fqdn",
    "::1",
    "",
    "192.168.32.0/24",
    "fe80::100/10",
    "42540766412265424405338506004571095040/64",
    True,
]


VALID_OUTPUT = [
    "192.24.2.1",
    "host.fqdn",
    "[::1]",
    "",
    "192.168.32.0/24",
    "[fe80::100]/10",
    "[2001:db8:32c:faad::]/64",
    True,
]


class TestIpWrap(unittest.TestCase):
    def setUp(self):
        pass

    def test_ipwrap_undefined_value(self):
        """Check ipwrap filter undefined value"""
        args = ["", AnsibleUndefined(name="my_ip"), ""]
        with pytest.raises(
            AnsibleFilterError,
            match="Unrecognized type <<class 'ansible.template.AnsibleUndefined'>> for ipwrap filter <value>",
        ):
            _ipwrap(*args)

    def test_valid_data_list(self):
        """Check passing valid argspec(list)"""
        args = ["", VALID_DATA, ""]
        result = _ipwrap(*args)
        print(result)
        self.assertEqual(result, VALID_OUTPUT)

    def test_valid_data_string(self):
        """Check passing valid argspec(string)"""

        args = ["", "::1", ""]
        result = _ipwrap(*args)
        self.assertEqual(result, "[::1]")



from __future__ import absolute_import, division, print_function


__metaclass__ = type

import subprocess

import pytest


def run(localhost_project, environment):
    __tracebackhide__ = True
    args = [
        "ansible-navigator",
        "run",
        str(localhost_project.playbook),
        "--ee",
        "false",
        "--mode",
        "stdout",
        "--pas",
        str(localhost_project.playbook_artifact),
        "--ll",
        "debug",
        "--lf",
        str(localhost_project.log_file),
        "--cdcp",
        str(localhost_project.collection_doc_cache),
        "-vvvv",
    ]
    process = subprocess.run(
        args=args,
        env=environment,
        stderr=subprocess.PIPE,
        stdout=subprocess.PIPE,
        check=False,
        shell=False,
    )
    if process.returncode:
        print(process.stdout.decode("utf-8"))
        print(process.stderr.decode("utf-8"))

        pytest.fail(reason=f"Integration test failed: {localhost_project.role}")


def test_integration(localhost_project, environment, monkeypatch):
    run(localhost_project, environment)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible.errors import AnsibleLookupError

from ansible_collections.ansible.utils.plugins.lookup.validate import LookupModule


DATA = {
    "GigabitEthernet0/0/0/0": {
        "auto_negotiate": False,
        "counters": {
            "in_crc_errors": 0,
            "in_errors": 0,
            "rate": {"in_rate": 0, "out_rate": 0},
        },
        "description": "configured using Ansible",
        "duplex_mode": "full",
        "enabled": True,
        "line_protocol": "up",
        "mtu": 1514,
        "oper_status": "down",
        "type": "GigabitEthernet",
    },
    "GigabitEthernet0/0/0/1": {
        "auto_negotiate": False,
        "counters": {
            "in_crc_errors": 10,
            "in_errors": 0,
            "rate": {"in_rate": 0, "out_rate": 0},
        },
        "description": "# interface is configures with Ansible",
        "duplex_mode": "full",
        "enabled": False,
        "line_protocol": "up",
        "mtu": 1514,
        "oper_status": "up",
        "type": "GigabitEthernet",
    },
}

CRITERIA_CRC_ERROR_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {
            "type": "object",
            "properties": {
                "counters": {"properties": {"in_crc_errors": {"type": "number", "maximum": 0}}},
            },
        },
    },
}

CRITERIA_ENABLED_CHECK = {
    "type": "object",
    "patternProperties": {"^.*": {"type": "object", "properties": {"enabled": {"enum": [True]}}}},
}

CRITERIA_OPER_STATUS_UP_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {
            "type": "object",
            "properties": {"oper_status": {"type": "string", "pattern": "up"}},
        },
    },
}

CRITERIA_IN_RATE_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {
            "type": "object",
            "properties": {
                "counters": {
                    "properties": {
                        "rate": {"properties": {"in_rate": {"type": "number", "maximum": 0}}},
                    },
                },
            },
        },
    },
}


class TestValidate(unittest.TestCase):
    def setUp(self):
        self._lp = LookupModule()

    def test_invalid_argspec(self):
        """Check passing invalid argspec"""

        # missing required arguments
        with self.assertRaises(AnsibleLookupError) as error:
            self._lp.run([DATA], {})
        self.assertIn("missing either 'data' or 'criteria' value", str(error.exception))

        terms = [DATA, [CRITERIA_IN_RATE_CHECK]]
        kwargs = {"engine": "ansible.utils.sample"}
        variables = {}
        with self.assertRaises(AnsibleLookupError) as error:
            self._lp.run(terms, variables, **kwargs)
        self.assertIn(
            "For engine 'ansible.utils.sample' error loading",
            str(error.exception),
        )

        terms = ["invalid data", [CRITERIA_IN_RATE_CHECK]]
        kwargs = {"engine": "ansible.utils.jsonschema"}
        variables = {}
        with self.assertRaises(AnsibleLookupError) as error:
            self._lp.run(terms, variables, **kwargs)
        self.assertIn("'data' option value is invalid", str(error.exception))

        terms = [DATA, "invalid criteria"]
        kwargs = {"engine": "ansible.utils.jsonschema"}
        variables = {}
        with self.assertRaises(AnsibleLookupError) as error:
            self._lp.run(terms, variables, **kwargs)
        self.assertIn("'criteria' option value is invalid", str(error.exception))

    def test_invalid_validate_plugin_config_options(self):
        """Check passing invalid validate plugin options"""

        terms = [
            DATA,
            [
                CRITERIA_CRC_ERROR_CHECK,
                CRITERIA_ENABLED_CHECK,
                CRITERIA_OPER_STATUS_UP_CHECK,
            ],
        ]
        kwargs = {"engine": "ansible.utils.jsonschema"}
        variables = {}
        result = self._lp.run(terms, variables, **kwargs)
        self.assertIn(
            "GigabitEthernet0/0/0/1.counters.in_crc_errors",
            result[0]["data_path"],
        )
        self.assertIn("GigabitEthernet0/0/0/1.enabled", result[1]["data_path"])
        self.assertIn("GigabitEthernet0/0/0/0.oper_status", result[2]["data_path"])

    def test_valid_data(self):
        """Check passing valid data as per criteria"""

        terms = [DATA, CRITERIA_IN_RATE_CHECK]
        kwargs = {"engine": "ansible.utils.jsonschema"}
        variables = {}
        result = self._lp.run(terms, variables, **kwargs)
        self.assertEqual(result, [])



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for ipwrap filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.filter.ip4_hex import _ip4_hex


class TestIpWrap(unittest.TestCase):
    def setUp(self):
        pass

    def test_valid_data_list(self):
        """Check passing valid argspec(list)"""
        args = ["", "192.168.1.5", ""]
        result = _ip4_hex(*args)
        print(result)
        self.assertEqual(result, "c0a80105")

    def test_valid_data_string(self):
        """Check passing valid argspec(string)"""

        args = ["", "192.168.1.5", ":"]
        result = _ip4_hex(*args)
        self.assertEqual(result, "c0:a8:01:05")

    def test_args(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = []
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            _ip4_hex(*args, **kwargs)
        self.assertIn("missing required arguments: arg", str(error.exception))



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for ipmath filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.filter.ipmath import _ipmath


class TestIpAddr(unittest.TestCase):
    def setUp(self):
        pass

    def test_find_next_fifth_address(self):
        """Get the next fifth address based on an IP address"""

        args = ["", "192.168.1.5", 5]
        result = _ipmath(*args)
        self.assertEqual(result, "192.168.1.10")

    def test_find_previous_fifth_address(self):
        """Get the previous fifth address"""

        args = ["", "192.168.1.5", -10]
        result = _ipmath(*args)
        self.assertEqual(result, "192.168.0.251")

    def test_find_next_fifth_address_cidr(self):
        """Get the next fifth address CIDR notation"""

        args = ["", "192.168.1.1/24", 5]
        result = _ipmath(*args)
        self.assertEqual(result, "192.168.1.6")

    def test_find_previous_fifth_address_cidr(self):
        """Get the previous fifth address CIDR notation"""

        args = ["", "192.168.1.6/24", -5]
        result = _ipmath(*args)
        self.assertEqual(result, "192.168.1.1")

    def test_find_next_fifth_address_ipv6(self):
        """Get the next fifth address in ipv6"""

        args = ["", "2001::1", 10]
        result = _ipmath(*args)
        self.assertEqual(result, "2001::b")

    def test_find_previous_fifth_address_ipv6(self):
        """Get the previous fifth address in ipv6"""

        args = ["", "2001::5", -10]
        result = _ipmath(*args)
        self.assertEqual(result, "2000:ffff:ffff:ffff:ffff:ffff:ffff:fffb")

    def test_invalid_data(self):
        """Check passing invalid data"""

        args = ["", "2001::1.999.0", 10]
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            _ipmath(*args, **kwargs)
        self.assertIn("You must pass a valid IP address", str(error.exception))



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for network_in_usable filter plugin
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import unittest

from ansible_collections.ansible.utils.plugins.filter.network_in_usable import _network_in_usable


class Test_Network_In_Usable(unittest.TestCase):
    def setUp(self):
        pass

    def test_network_in_usable_filter_1(self):
        """network_in_usable filter"""
        args = ["", "192.168.0.0/24", "192.168.0.1"]
        result = _network_in_usable(*args)
        self.assertEqual(result, True)

    def test_network_in_usable_filter_2(self):
        """network_in_usable filter"""
        args = ["", "192.168.0.0/24", "192.168.0.255"]
        result = _network_in_usable(*args)
        self.assertEqual(result, False)

    def test_network_in_usable_filter_3(self):
        """network_in_usable filter"""
        args = ["", "192.168.0.0/16", "192.168.0.255"]
        result = _network_in_usable(*args)
        self.assertEqual(result, True)
